using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Phasma
{
    /// <summary>
    /// RLWindow that displays a set of commands for the user to click on
    /// </summary>
    public class RLCommand : RLWindow
    {
        /// <summary>
        /// Basic constructor for RLCommand Objects.
        /// </summary>
        /// <param name="iX">Starting x-coordinate, in pixels.</param>
        /// <param name="iY">Starting y-coordinate, in pixels.</param>
        /// <param name="iW">Starting width, in pixels.</param>
        /// <param name="iH">Starting height, in pixels.</param>
        public RLCommand(int iX, int iY, int iW, int iH)
            : base(iX, iY, iW, iH, -1, -1)
        { }

        /// <summary>
        /// Structure to hold a set of commands to be displayed by an RLCommand object.
        /// </summary>
        public struct CommandSet
        {
            /// <summary>
            /// Visual names of the commands in the set.
            /// </summary>
            public string[] DisplayNames;
            /// <summary>
            /// Strings that will be sent to DoCommand when a command is clicked.
            /// </summary>
            public string[] Commands;
            /// <summary>
            /// Color of the commands, when displayed.
            /// </summary>
            public Color[] Colors;
        }

        /// <summary>
        /// Handler for the event triggered when the user clicks a command.
        /// </summary>
        /// <param name="outCmd">String from Commands[] for the command the user clicked.</param>
        public delegate void DoCommandHandler(string outCmd);
        /// <summary>
        /// Event triggered when the user clicks a command.
        /// </summary>
        public event DoCommandHandler DoCommand;

        private Dictionary<string, CommandSet> myCommandSets = new Dictionary<string, CommandSet>();
        private CommandSet currentCommandSet;

        /// <summary>
        /// Whether or not commands should be highlit on mouseover.
        /// </summary>
        public bool UseHighlights { get; set; }

        /// <summary>
        /// Color in which to highlight the menu options, if highlights are turned on.
        /// </summary>
        public Color HighlightColor { get; set; }

        /// <summary>
        /// Adds a new command set to this object.
        /// </summary>
        /// <param name="state">Sets what state of the program this command set should be associated with.</param>
        /// <param name="newCS">New CommandSet to be added.</param>
        public void AddCommandSet(string state, CommandSet newCS)
        {
            if (newCS.DisplayNames.Length != newCS.Commands.Length || newCS.DisplayNames.Length != newCS.DisplayNames.Length)
                throw new Exception("Bad CommandSet- mismatched numbers of params");
            myCommandSets.Add(state, newCS);
        }

        /// <summary>
        /// Causes this object to switch to the command set associated with the given state.
        /// </summary>
        /// <param name="inState">String describing which state we should be switching to.</param>
        public void SwitchToState(string state)
        {
            if (!myCommandSets.ContainsKey(state))
                return;
            currentCommandSet = myCommandSets[state];
            DrawMe();
        }

        private int GetCommandFromPoint(int X, int Y)
        {
            int xO = 1, yO = 0;

            for (int i = 0; i < currentCommandSet.DisplayNames.Length; i++)
            {
                if (xO + currentCommandSet.DisplayNames[i].Length > CellsWide)
                {
                    yO += 1;
                    xO = 1;
                }
                xO += currentCommandSet.DisplayNames[i].Length + 1;
                if (yO == Y && xO > X)
                {
                    return i;
                }
            }

            return -1;
        }

        private Rectangle GetRegionForCommand(int commandIndex)
        {
            if (currentCommandSet.DisplayNames == null)
                return new Rectangle(0, 0, 0, 0);

            int xO = 1, yO = 0;

            for (int i = 0; i <= commandIndex; i++)
            {
                if (xO + currentCommandSet.DisplayNames[i].Length > CellsWide)
                {
                    yO += 1;
                    xO = 1;
                }
                if (i != commandIndex)
                    xO += currentCommandSet.DisplayNames[i].Length + 1;
            }

            return new Rectangle(xO, yO, currentCommandSet.DisplayNames[commandIndex].Length, 1);
        }

        internal override void RaiseClick(int X, int Y, System.Windows.Forms.MouseButtons MB)
        {
            if (MB != System.Windows.Forms.MouseButtons.Left)
                return;
            if (currentCommandSet.DisplayNames == null)
                return;

            if (DoCommand != null)
                DoCommand(currentCommandSet.Commands[GetCommandFromPoint(X, Y)]);
        }

        /// <summary>
        /// Returns the number of cells vertically that this object currently requires.
        /// </summary>
        /// <returns>Number of cells vertically that this object currently requires.</returns>
        public int HeightNeeded()
        {
            if (currentCommandSet.DisplayNames == null)
                return 0;

            int xO = 1, yO = 0;

            for (int i = 0; i < currentCommandSet.DisplayNames.Length; i++)
            {
                if (xO + currentCommandSet.DisplayNames[i].Length > CellsWide)
                {
                    yO += 1;
                    xO = 1;
                }
                xO += currentCommandSet.DisplayNames[i].Length + 1;
            }

            return yO + 1;
        }

        private void DrawMe()
        {
            if (currentCommandSet.DisplayNames == null)
                return;

            for (int x = 0; x < CellsWide; x++)
                for (int y = 0; y < CellsTall; y++)
                    SetCell(x, y, ' ', false);

            int xO = 1, yO = 0;

            for (int i = 0; i < currentCommandSet.DisplayNames.Length; i++)
            {
                if (xO + currentCommandSet.DisplayNames[i].Length > CellsWide)
                {
                    yO += 1;
                    xO = 1;
                }
                DrawString(currentCommandSet.DisplayNames[i], xO, yO, currentCommandSet.Colors[i], Color.Black, false);
                xO += currentCommandSet.DisplayNames[i].Length + 1;
            }

            Update();
        }

        internal override void RaiseMouseMove(int X, int Y)
        {
            int commandToHighlight = GetCommandFromPoint(X, Y);
            if (commandToHighlight >= 0)
            {
                Rectangle highlightRect = GetRegionForCommand(commandToHighlight);
                HighlightCellRegion(highlightRect.X, highlightRect.Y, highlightRect.Width, highlightRect.Height, HighlightColor);
            }
            base.RaiseMouseMove(X, Y);
        }
    }
}
