using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace Phasma
{
    /// <summary>
    /// Primary display object for Phasma. All other visual objects are contained by and drawn on this one.
    /// </summary>
    public partial class RLDisplay : UserControl
    {
        public RLDisplay()
        {
            InitializeComponent();
        }

        private List<RLWindow> myWindows = new List<RLWindow>();

        /// <summary>
        /// Returns the most recently popped-up menu.
        /// </summary>
        public RLMenu ActiveMenu = null;

        private Graphics myGraphics = null;
        public Graphics MyGraphics
        {
            get
            {
                if (myGraphics == null)
                    myGraphics = CreateGraphics();
                return myGraphics;
            }
        }

        private bool myUseMouse = false;
        /// <summary>
        /// Gets or sets whether or not this control's windows accept mouse events.
        /// </summary>
        [Browsable(true)]
        public bool AllowMouse
        {
            get { return myUseMouse; }
            set { myUseMouse = value; }
        }

        /// <summary>
        /// Adds the given RLWindow to the display.
        /// </summary>
        /// <param name="inWindow">RLWindow to be added.</param>
        public void AddWindow(RLWindow inWindow)
        {
            if (myWindows.Contains(inWindow))
                return;
            myWindows.Add(inWindow);
            inWindow.myParent = this;
            inWindow.Font = Font;
            if (inWindow != ActiveMenu)
                MoveToTop(ActiveMenu, false);
            Refresh();
        }

        /// <summary>
        /// Removes the given RLWindow from the display.
        /// </summary>
        /// <param name="inWindow">RLWindow to be removed.</param>
        public void RemWindow(RLWindow inWindow)
        {
            if (!myWindows.Contains(inWindow))
                return;
            myWindows.Remove(inWindow);
            inWindow.myParent = null;
            RedrawWindows();
        }

        /// <summary>
        /// Checks to see if this display object contains the given RLWindow.
        /// </summary>
        /// <param name="inWindow">RLWindow to search for.</param>
        /// <returns>Whether or not this display object contains the given RLWindow.</returns>
        public bool HasWindow(RLWindow inWindow)
        {
            return myWindows.Contains(inWindow);
        }

        /// <summary>
        /// Forces the display to immediately redraw all windows associated with it.
        /// </summary>
        public void RedrawWindows()
        {
            RedrawWindows(MyGraphics);
        }

        private void RedrawWindows(Graphics G)
        {
            Image buf = new Bitmap(Width, Height);
            Graphics tempG = Graphics.FromImage(buf);

            tempG.FillRectangle(Brushes.Black, 0, 0, Width, Height);
            foreach (RLWindow W in myWindows)
                W.DrawWindow(tempG);

            G.DrawImage(buf, 0, 0);
        }

        internal bool drawingLocked = false;

        /// <summary>
        /// Prevents this display from doing any drawing until UnlockDrawing() is called.
        /// </summary>
        public void LockDrawing()
        {
            drawingLocked = true;
        }

        /// <summary>
        /// Allows this display to begin drawing again, after LockDrawing() is called. Also causes an immediate redraw.
        /// </summary>
        public void UnlockDrawing()
        {
            drawingLocked = false;
            RedrawWindows();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            RedrawWindows(e.Graphics);
        }

        /// <summary>
        /// Opens a given RLMenu object as a popped-up window.
        /// </summary>
        /// <param name="inMenu">RLMenu object to be popped up.</param>
        public void PopMenu(RLMenu inMenu)
        {
            inMenu.PreviousMenu = ActiveMenu;
            ActiveMenu = inMenu;
            AddWindow(inMenu);
        }

        /// <summary>
        /// Pushes a given RLWindow object to the top of the display.
        /// </summary>
        /// <param name="inWindow">RLWindow to be moved to the top.</param>
        public void MoveToTop(RLWindow inWindow)
        {
            MoveToTop(inWindow, true);
        }

        private void MoveToTop(RLWindow inWindow, bool redraw)
        {
            if (myWindows.Contains(inWindow))
            {
                myWindows.Remove(inWindow);
                myWindows.Add(inWindow);
                if (redraw)
                    Refresh();
            }
        }

        /// <summary>
        /// Resets the graphics object.
        /// </summary>
        public void ResetGraphics()
        {
            myGraphics = CreateGraphics();
        }

        private void RLDisplay_MouseClick(object sender, MouseEventArgs e)
        {
            if (!AllowMouse)
                return;

            for (int i = myWindows.Count - 1; i >= 0; i--)
            {
                RLWindow R = myWindows[i];
                Rectangle RBounds = new Rectangle(R.X, R.Y, R.Width, R.Height);
                if (RBounds.Contains(e.X, e.Y))
                {
                    R.RaiseClick((e.X - R.X) / R.CellW, (e.Y - R.Y) / R.CellH, e.Button);
                    break;
                }
            }
        }

        private void RLDisplay_MouseMove(object sender, MouseEventArgs e)
        {
            if (!AllowMouse)
                return;

            if (ActiveMenu != null)
            {
                RLWindow R = ActiveMenu;
                Rectangle RBounds = new Rectangle(R.X, R.Y, R.Width, R.Height);
                if (RBounds.Contains(e.X, e.Y))
                {
                    R.RaiseMouseMove((e.X - R.X) / R.CellW, (e.Y - R.Y) / R.CellH);
                }
            }
            else
            {

                for (int i = myWindows.Count - 1; i >= 0; i--)
                {
                    RLWindow R = myWindows[i];
                    Rectangle RBounds = new Rectangle(R.X, R.Y, R.Width, R.Height);
                    if (RBounds.Contains(e.X, e.Y))
                    {
                        R.RaiseMouseMove((e.X - R.X) / R.CellW, (e.Y - R.Y) / R.CellH);
                        break;
                    }
                }
            }
        }

        protected override bool IsInputKey(Keys keyData)
        {
            return true;
        }
    }
}
