using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace Phasma
{
    /// <summary>
    /// Takes keyboard input, transforms it into command strings based on XML keymaps.
    /// </summary>
    public class RLKeybindController
    {        
        private bool myHandlingKeys = true;        
        internal bool HandlingKeys
        {
            get { return myHandlingKeys; }
            set { myHandlingKeys = value; }
        }

        /// <summary>
        /// Create an RLKeybindController object based on a given set of keybindings.
        /// </summary>
        /// <param name="inDoc">XmlDocument containing the appropriate keymap.</param>
        public RLKeybindController(XmlDocument inDoc)
        {
            KeysConverter KC = new KeysConverter();

            XmlNode Root = inDoc.GetElementsByTagName("KeyBindings")[0];
            if (Root == null)
                return;

            foreach (XmlElement X in Root.ChildNodes)
            {
                keybindDef D = new keybindDef();
                D.Code = (Keys)KC.ConvertFromString(X.GetAttribute("Key"));
                if (X.HasAttribute("Alt"))
                    D.isAlt = bool.Parse(X.GetAttribute("Alt"));
                if (X.HasAttribute("Ctrl"))
                    D.isCtrl = bool.Parse(X.GetAttribute("Ctrl"));
                if (X.HasAttribute("Shift"))
                    D.isShift = bool.Parse(X.GetAttribute("Shift"));
                D.myState = X.GetAttribute("State");
                D.Command = X.GetAttribute("Command");

                myBindings.Add(D);
            }            
        }

        /// <summary>
        /// Takes in a KeyEventArgs and a state and (if appropriate) causes DoCommand to fire with the appropriate string.
        /// </summary>
        /// <param name="e">KeyEventArgs object to be processed.</param>
        /// <param name="myState">Current application state.</param>
        public void HandleKey(KeyEventArgs e, string myState)
        {
            if (HandlingKeys)
            {
                Keys EKC = e.KeyCode;

                //numpad handling
                if (EKC == Keys.NumPad7)
                    EKC = Keys.Home;
                else if (EKC == Keys.NumPad8)
                    EKC = Keys.Up;
                else if (EKC == Keys.NumPad9)
                    EKC = Keys.PageUp;
                else if (EKC == Keys.NumPad4)
                    EKC = Keys.Left;
                else if (EKC == Keys.NumPad5)
                    EKC = Keys.Clear;
                else if (EKC == Keys.NumPad6)
                    EKC = Keys.Right;
                else if (EKC == Keys.NumPad1)
                    EKC = Keys.End;
                else if (EKC == Keys.NumPad2)
                    EKC = Keys.Down;
                else if (EKC == Keys.NumPad3)
                    EKC = Keys.PageDown;


                for (int i = 0; i < myBindings.Count; i++)
                {
                    keybindDef K = myBindings[i];
                    if (K.Code == EKC && K.myState == myState)
                        if (K.isAlt == e.Alt && K.isCtrl == e.Control && K.isShift == e.Shift)
                        {
                            if (DoCommand != null)
                                DoCommand(K.Command);
                            break;
                        }

                }
            }
        }

        /// <summary>
        /// Given a command, returns what keycode and modifiers are associated with that command.
        /// Will throw an exception if the command is not currently bound.
        /// </summary>
        /// <param name="command">Command string to search for.</param>
        /// <param name="ctrl">Whether or not the mapped key requires ctrl to be pressed.</param>
        /// <param name="shift">Whether or not the mapped key requires shift to be pressed.</param>
        /// <param name="alt">Whether or not the mapped key requires alt to be pressed.</param>
        /// <returns>Keys object that has been bound to the given command.</returns>
        public Keys GetKey(string command, out bool ctrl, out bool shift, out bool alt)
        {
            for (int i = 0; i < myBindings.Count; i++)
            {
                keybindDef K = myBindings[i];
                if (K.Command == command)
                {
                    ctrl = K.isCtrl;
                    shift = K.isShift;
                    alt = K.isAlt;
                    return K.Code;
                }
            }
            throw new Exception("Invalid command.");
        }

        public delegate void DoCommandHandler(string outCmd);
        /// <summary>
        /// Event triggered when this object is given a key and state that correspond to a mapped command.
        /// </summary>
        public event DoCommandHandler DoCommand;

        private List<keybindDef> myBindings = new List<keybindDef>();


    }
}
