using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Phasma
{
    /// <summary>
    /// RLWindow that consists of a scrolling message history object.
    /// </summary>
    public class RLMessage : RLWindow
    {
        /// <summary>
        /// Basic constructor for RLMessage Objects.
        /// </summary>
        /// <param name="iX">Starting x-coordinate, in pixels.</param>
        /// <param name="iY">Starting y-coordinate, in pixels.</param>
        /// <param name="iW">Starting width, in pixels.</param>
        /// <param name="iH">Starting height, in pixels.</param>
        public RLMessage(int iX, int iY, int iW, int iH)
            : base(iX, iY, iW, iH, -1, -1)
        {
        }

        private List<KeyValuePair<string, Color>> myMessages = new List<KeyValuePair<string, Color>>();

        private char[] breakingChars = new char[] { ' ' };

        /// <summary>
        /// Clears all messages in the history and updates the display.
        /// </summary>
        public void ClearMessages()
        {
            myMessages.Clear();
            DrawToDisplay();
            Update();
        }

        private int myHistoryDepth = 100;
        /// <summary>
        /// Gets or sets the number of messages back to store in the history at any one time.
        /// Default is 100.
        /// </summary>
        public int HistoryDepth
        {
            get { return myHistoryDepth; }
            set { myHistoryDepth = value; }
        }	

        /// <summary>
        /// Adds a message with a given text and color to the history, and updates the display.
        /// </summary>
        /// <param name="inString">Text of the message to be added.</param>
        /// <param name="msgColor">Color of the message to be added.</param>
        public void AddMessage(string inString, Color msgColor)
        {
            //while the string is longer than we can display in one line...
            while (inString.Length >= CellsWide - 1)
            {
                string addString = "";
                //trim it at breaking characters
                while (true)
                {
                    int nIdx = inString.IndexOfAny(breakingChars, addString.Length);
                    if (nIdx < 0 && addString.Length == 0)
                    {
                        addString = inString.Substring(0, CellsWide - 2);
                        break;
                    }
                    else if (nIdx < 0)
                        nIdx = inString.Length - 1;

                    string nextString = inString.Substring(addString.Length, nIdx - addString.Length + 1);
                    if (addString.Length + nextString.Length > CellsWide - 1)
                        break;
                    else if (nextString.Length == 0)
                        break;
                    else
                        addString += nextString;
                }
                if (addString.Length == 0)
                    continue;

                if (myMessages.Count == HistoryDepth)
                    myMessages.RemoveAt(HistoryDepth - 1);
                myMessages.Insert(0, new KeyValuePair<string, Color>(addString, msgColor));
                inString = inString.Remove(0, addString.Length);
                DrawToDisplay();
            }            
            if (inString.Length > 0)
            {
                if (myMessages.Count == HistoryDepth)
                    myMessages.RemoveAt(HistoryDepth - 1);
                myMessages.Insert(0, new KeyValuePair<string, Color>(inString, msgColor));
                DrawToDisplay();
            }
            Update();
        }

        internal override void RenderCachedImage()
        {
            if (Width <= 0 || Height <= 0)
                return;
            Image img = new Bitmap(Width, Height, System.Drawing.Imaging.PixelFormat.Format16bppRgb565);
            Graphics myG = Graphics.FromImage(img);
            myG.FillRectangle(Brushes.Black, 0, 0, Width, Height);

            if (myMessages.Count <= 0)
            {
                myCachedImage = img;
                return;
            }

            for (int y = 0; y < CellsTall; y++)
            {
                if (CellsTall - 1 - y < myMessages.Count)
                {
                    
                    KeyValuePair<string, Color> K = myMessages[CellsTall - y - 1];
                    myG.DrawString(K.Key, Font, new SolidBrush(K.Value), 0, y * CellH);
                    
                }
            }
            myCachedImage = img;
        }

        private void DrawToDisplay()
        {
            myCachedImage = null;
            DrawWindow();           
        }

        /// <summary>
        /// Adds a message with a given text to the history in white, and updates the display.
        /// </summary>
        /// <param name="inString">Text of the message to be added.</param>        
        public void AddMessage(string inString)
        {
            AddMessage(inString, Color.White);
        }

        /// <summary>
        /// Returns a string array containing the entire message history.
        /// </summary>
        /// <returns>Contents of the message history.</returns>
        public string[] GetMessages()
        {
            List<string> rV = new List<string>();
            foreach (KeyValuePair<string, Color> K in myMessages)
                rV.Add(K.Key);
            return rV.ToArray();
        }
    }
}
