using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Phasma
{
    /// <summary>
    /// RLWindow used to display a map onscreen. Contains functionality for a scrolling viewable window area.
    /// This means that any calls to SetCell, StackCell, or UpdateCell on this object can be called with map-cell coordinates, 
    /// and they will be translated into screen-cell coordinates.
    /// </summary>
    public class RLScreen : RLWindow
    {
        /// <summary>
        /// Basic constructor for RLScreen Objects.
        /// </summary>
        /// <param name="iX">Starting x-coordinate, in pixels.</param>
        /// <param name="iY">Starting y-coordinate, in pixels.</param>
        /// <param name="iW">Starting width, in pixels.</param>
        /// <param name="iH">Starting height, in pixels.</param>
        /// <param name="WiX">Starting x-coordinate of the viewable window, in cells.</param>
        /// <param name="WiY">Starting y-coordinate of the viewable window, in cells.</param>
        public RLScreen(int iX, int iY, int iW, int iH, int WiX, int WiY)
            : base(iX, iY, iW, iH, -1, -1)
        {
            WindowLoc = new Point(WiX, WiY);
        }

        private Point myWindowLoc;
        /// <summary>
        /// Gets or sets the coordinate of the upper-right corner of the viewable window, in cells.
        /// </summary>
        public Point WindowLoc
        {
            get { return myWindowLoc; }
            set { myWindowLoc = value; }
        }

        internal override void RaiseClick(int X, int Y, System.Windows.Forms.MouseButtons MB)
        {
            base.RaiseClick(X + WindowLoc.X, Y + WindowLoc.Y, MB);
        }

        /// <summary>
        /// Sets what tile should be displayed at a given coordinate.
        /// </summary>
        /// <param name="x">X-coordinate to set, in cells.</param>
        /// <param name="y">Y-coordinate to set, in cells.</param>
        /// <param name="T">Tile to set.</param>
        /// <param name="updateNow">Determines whether the display should be updated at this time or not.</param>
        public override void SetCell(int x, int y, Tile T, bool updateNow)
        {
            base.SetCell(x - WindowLoc.X, y - WindowLoc.Y, T, updateNow);
        }

        /// <summary>
        /// Stacks on another tile at a given coordinate.
        /// </summary>
        /// <param name="x">X-coordinate to stack on at, in cells.</param>
        /// <param name="y">Y-coordinate to stack on at, in cells.</param>
        /// <param name="T">Tile to stack on.</param>
        /// <param name="updateNow">Determines whether the display should be updated at this time or not.</param>
        public override void StackCell(int x, int y, Tile T, bool updateNow)
        {
            base.StackCell(x - WindowLoc.X, y - WindowLoc.Y, T, updateNow);
        }

        /// <summary>
        /// Immediately updates the cell at the given coordinates.
        /// </summary>
        /// <param name="x">X-coordinate to update at, in cells.</param>
        /// <param name="y">Y-coordinate to update at, in cells.</param>
        public override void UpdateCell(int x, int y)
        {
            base.UpdateCell(x - WindowLoc.X, y - WindowLoc.Y);
        }

        /// <summary>
        /// Clears the given cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to be cleared, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to be cleared, in cells.</param>
        /// <param name="updateNow">Determines whether the display should be updated immediately.</param>
        public override void ClearCell(int x, int y, bool updateNow)
        {
            base.ClearCell(x - WindowLoc.X, y - WindowLoc.Y, updateNow);
        }

        /// <summary>
        /// Sets the highlight for this screen
        /// </summary>
        /// <param name="x">X coordinate of the rectangle to highlight</param>
        /// <param name="y">Y coordinate of the rectangle to highlight</param>
        /// <param name="width">Width of the rectangle to highlight</param>
        /// <param name="height">Height of the rectangle to highlight</param>
        /// <param name="highlightColor">Color to highlight in</param>
        public override void HighlightCellRegion(int x, int y, int width, int height, Color highlightColor)
        {
            base.HighlightCellRegion(x - WindowLoc.X, y - WindowLoc.Y, width, height, highlightColor);
        }

        internal override void RaiseMouseMove(int X, int Y)
        {
            base.RaiseMouseMove(X + WindowLoc.X, Y + WindowLoc.Y);
        }
    }
}
