using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Phasma
{
    /// <summary>
    /// RLWindow that can display information lines in several different formats.
    /// </summary>
    public class RLStats : RLWindow
    {
        /// <summary>
        /// Basic constructor for RLStats Objects.
        /// </summary>
        /// <param name="iX">Starting x-coordinate, in pixels.</param>
        /// <param name="iY">Starting y-coordinate, in pixels.</param>
        /// <param name="iW">Starting width, in pixels.</param>
        /// <param name="iH">Starting height, in pixels.</param>
        public RLStats(int iX, int iY, int iW, int iH)
            : base(iX, iY, iW, iH, -1, -1)
        {
        }

        private Dictionary<string, statDef> myStats = new Dictionary<string, statDef>();

        /// <summary>
        /// Allows you to get the individual statDef objects contained within this RLStats.
        /// </summary>
        /// <param name="Key">Key of the statDef you want to retrieve.</param>
        /// <returns>The statDef with the given key.</returns>
        public statDef this[string Key]
        {
            get
            {
                if (!myStats.ContainsKey(Key))
                    return null;
                return myStats[Key];
            }
            set
            {
                if (!myStats.ContainsKey(Key))
                    return;
                myStats[Key] = value;
            }
        }

        private bool locked = false;

        /// <summary>
        /// Prevents this RLStats from redrawing until UnlockDrawing() is called.
        /// </summary>
        public void LockDrawing()
        {
            locked = true;
        }

        /// <summary>
        /// Allows this RLStats to redraw again, and calls for an immediate update.
        /// </summary>
        public void UnlockDrawing()
        {
            locked = false;
            Update();
        }

        /// <summary>
        /// Adds a new statDef object to this RLStats object and updates the display with it.
        /// </summary>
        /// <param name="inStat">statDef to be added.</param>
        public void AddStat(statDef inStat)
        {
            if (myStats.ContainsKey(inStat.Key))
                return;
            myStats.Add(inStat.Key, inStat);
            inStat.ClearMe += new statDef.statEvent(inStat_ClearMe);
            inStat.DrawMe += new statDef.statEvent(inStat_DrawMe);
            inStat_DrawMe(inStat.Key);
        }

        private void inStat_DrawMe(string outKey)
        {
            statDef S = myStats[outKey];

            if (!S.Visible)
            {
                if (!locked)
                    Update();
                return;
            }

            int xOff = 0;

            if (S.StatType == statDef.StatTypes.Output || S.StatType == statDef.StatTypes.Bar)
            {
                DrawString(S.Key + ": ", S.X, S.Y, S.Color1, Color.Black, false);
                xOff += S.Key.Length + 2;
            }

            if (S.StatType == statDef.StatTypes.Output || S.StatType == statDef.StatTypes.KeylessOutput)
            {
                DrawString(S.Value, S.X + xOff, S.Y, S.Color2, Color.Black, false);
            }
            else if (S.StatType == statDef.StatTypes.Bar)
            {
                try
                {
                    string[] vals = S.Value.Split('/');
                    if (vals[0] == "" || vals[1] == "")
                    {
                        DrawString("<??????????> (?/?)", S.X + xOff, S.Y, S.Color2, Color.Black, false);
                        return;
                    }
                    float A = float.Parse(vals[0]);
                    float B = float.Parse(vals[1]);
                    int numEmpty = 10 - Convert.ToInt32(Math.Min(Math.Ceiling(A / B * 10), 10));
                    string outS = "<";
                    for (int i = 0; i < 10; i++)
                        if (i >= numEmpty)
                            outS += "#";
                        else
                            outS += "-";

                    outS += "> (" + S.Value + ")";
                    DrawString(outS, S.X + xOff, S.Y, S.Color2, Color.Black, false);
                }
                catch
                {
                    DrawString("<??????????> (?/?)", S.X + xOff, S.Y, S.Color2, Color.Black, false);
                }
            }

            if (S.StatType == statDef.StatTypes.FlagList)
            {
                int yOff = 0;

                foreach (statDef F in ((flagListDef)S).Flags)
                {
                    if (F.Visible)
                    {
                        DrawString(F.Value, S.X, S.Y + yOff, F.Color1, Color.Black, false);
                        yOff++;
                    }
                }
            }

            if (!locked) Update();
        }

        private void inStat_ClearMe(string outKey)
        {
            statDef S = myStats[outKey];

            if (!S.Visible) return;

            if (S.StatType != statDef.StatTypes.FlagList)
            {
                for (int i = 0; i < S.Length; i++)
                    SetCell(S.X + i, S.Y, ' ', false);

            }
            else
            {
                int yOff = 0;
                foreach (statDef F in ((flagListDef)S).Flags)
                {
                    if (F.Visible)
                    {
                        for (int i = 0; i < F.Length; i++)
                            SetCell(S.X + i, S.Y + yOff, ' ', false);
                        yOff++;
                    }
                }

            }
        }

        /// <summary>
        /// Removes the statDef with the given key from this RLStats object.
        /// </summary>
        /// <param name="Key">Key of the statDef to be removed.</param>
        public void RemoveStat(string Key)
        {
            if (!myStats.ContainsKey(Key))
                return;
            statDef S = myStats[Key];
            S.ClearMe -= inStat_ClearMe;
            S.DrawMe -= inStat_DrawMe;
            inStat_ClearMe(Key);
            myStats.Remove(Key);
            Update();
        }
    }
}
