using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Phasma
{
    /// <summary>
    /// Abstract base class for anything that can be shown on an RLDisplay.
    /// </summary>
    public abstract class RLWindow
    {
        public delegate void MouseClickHandler(int cellX, int cellY, System.Windows.Forms.MouseButtons Buttons);
        /// <summary>
        /// Event triggered by the user clicking on this window.
        /// </summary>
        public event MouseClickHandler MouseClick;

        internal virtual void RaiseClick(int X, int Y, System.Windows.Forms.MouseButtons MB)
        {
            if (MouseClick != null)
                MouseClick(X, Y, MB);
        }

        public delegate void MouseMoveHandler(int cellX, int cellY);
        /// <summary>
        /// Event triggered by the user clicking on this window.
        /// </summary>
        public event MouseMoveHandler MouseMove;

        internal virtual void RaiseMouseMove(int X, int Y)
        {
            if (MouseMove != null)
                MouseMove(X, Y);
        }

        /// <summary>
        /// Basic constructor for RLWindow objects.
        /// </summary>
        public RLWindow()
        {
        }

        /// <summary>
        /// Basic constructor for RLWindow Objects.
        /// </summary>
        /// <param name="iX">Starting x-coordinate, in pixels.</param>
        /// <param name="iY">Starting y-coordinate, in pixels.</param>
        /// <param name="iW">Starting width, in pixels.</param>
        /// <param name="iH">Starting height, in pixels.</param>
        /// <param name="WiX">Initial width of the cells, in pixels.</param>
        /// <param name="WiY">Initial height of the cells, in pixels.</param>
        public RLWindow(int iX, int iY, int iWidth, int iHeight, int cellW, int cellH)
        {
            myX = iX;
            myY = iY;
            myWidth = iWidth;
            myHeight = iHeight;
            myCellH = cellH;
            myCellW = cellW;
            Font = new Font("Courier New", 12, FontStyle.Bold);
        }

        /// <summary>
        /// RLDisplay object that this object has been added to.
        /// </summary>
        public RLDisplay myParent;

        private bool myUseImages;
        /// <summary>
        /// Gets or sets whether or not this window draws using images.
        /// </summary>
        public bool UseImages
        {
            get { return myUseImages; }
            set { myUseImages = value; }
        }

        internal Dictionary<Point, cellDef> myDisplay = new Dictionary<Point, cellDef>();

        protected Image myCachedImage = null;

        private Rectangle? myHighlight = null;
        private Color? myHighlightColor = null;

        private int myX;
        /// <summary>
        /// Gets or sets the X-coordinate of this window, in pixels.
        /// </summary>
        public virtual int X
        {
            get { return myX; }
            set
            {
                myX = value;
                FlushWindow();
            }
        }

        private int myY;
        /// <summary>
        /// Gets or sets the Y-coordinate of this window, in pixels.
        /// </summary>
        public virtual int Y
        {
            get { return myY; }
            set
            {
                myY = value;
                FlushWindow();
            }
        }

        private int myWidth;
        /// <summary>
        /// Gets or sets the width of this window, in pixels.
        /// </summary>
        public virtual int Width
        {
            get { return Math.Max(myWidth, 1); }
            set
            {
                myWidth = value;
                FlushWindow();
            }
        }

        private int myHeight;
        /// <summary>
        /// Gets or sets the height of this window, in pixels.
        /// </summary>
        public virtual int Height
        {
            get { return Math.Max(myHeight, 1); }
            set
            {
                myHeight = value;
                FlushWindow();
            }
        }

        protected int myCellW;
        /// <summary>
        /// Gets or sets the width of this window's cells, in pixels. Note that if this window is not using images, this value is set by the font.
        /// </summary>
        public virtual int CellW
        {
            get
            {
                if (!UseImages)
                    return FontCellSize.Width + HorizontalCellPadding;
                return myCellW;
            }
            set
            {
                myCellW = value;
                FlushWindow();
            }
        }

        protected int myCellH;
        /// <summary>
        /// Gets or sets the height of this window's cells, in pixels. Note that if this window is not using images, this value is set by the font.
        /// </summary>
        public virtual int CellH
        {
            get
            {
                if (!UseImages)
                    return FontCellSize.Height + VerticalCellPadding;
                return myCellH;
            }
            set
            {
                myCellH = value;
                FlushWindow();
            }
        }

        /// <summary>
        /// Gets how many cells wide this window is.
        /// </summary>
        public int CellsWide
        {
            get
            {
                return Convert.ToInt32(Math.Floor(Width / (double)CellW));
            }
        }

        /// <summary>
        /// Gets how many cells tall this window is.
        /// </summary>
        public int CellsTall
        {
            get
            {
                return Convert.ToInt32(Math.Floor(Height / (double)CellH));
            }
        }

        /// <summary>
        /// Gets the bounds of this window, in pixels.
        /// </summary>
        public Rectangle Bounds
        {
            get
            {
                return new Rectangle(X, Y, Width, Height);
            }
        }

        /// <summary>
        /// Draws this window using its parent's default graphics object.
        /// </summary>
        internal void DrawWindow()
        {
            if (myParent == null)
                return;
            DrawWindow(myParent.MyGraphics);
        }

        /// <summary>
        /// Draws this window using the given graphics object.
        /// </summary>
        /// <param name="G">Graphics object with which to draw this window.</param>
        internal void DrawWindow(Graphics G)
        {
            if (myParent == null)
                return;
            if (myParent.drawingLocked)
                return;
            if (myCachedImage == null)
                RenderCachedImage();
            if (myCachedImage != null)
                G.DrawImage(myCachedImage, X, Y);
        }

        /// <summary>
        /// Redraws this window onto the internal cache image.
        /// </summary>
        internal virtual void RenderCachedImage()
        {
            Image img = new Bitmap(Width, Height, UseImages ? System.Drawing.Imaging.PixelFormat.Format32bppPArgb : System.Drawing.Imaging.PixelFormat.Format16bppRgb565);
            Graphics myG = Graphics.FromImage(img);
            myG.FillRectangle(Brushes.Black, 0, 0, Width, Height);
            Rectangle myBounds = Bounds;
            int cW = CellsWide;
            int cT = CellsTall;
            List<Point> dumpList = new List<Point>();
            foreach (KeyValuePair<Point, cellDef> K in myDisplay)
            {
                cellDef CD = K.Value;
                if (CD.X > cW)
                    dumpList.Add(K.Key);
                else if (CD.Y > cT)
                    dumpList.Add(K.Key);
                else
                    CD.DrawCell(myG, Font, 0, 0, CellW, CellH, OffsetLeft, OffsetTop, UseImages);
            }
            foreach (Point P in dumpList)
                myDisplay.Remove(P);

            RenderHighlight(myG);

            myCachedImage = img;
        }

        private void RenderHighlight(Graphics myG)
        {
            if (myHighlight != null && myHighlightColor != null)
            {
                Color localHighlightColor = myHighlightColor.Value;
                Rectangle localHighlight = myHighlight.Value;
                myG.DrawRectangle(new Pen(localHighlightColor, 1), localHighlight.X * CellW, localHighlight.Y * CellH, localHighlight.Width * CellW - 1, localHighlight.Height * CellH - 1);
            }
        }

        private Font myFont;
        /// <summary>
        /// Gets or sets the font used by this window.
        /// </summary>
        public Font Font
        {
            get { return myFont; }
            set
            {
                int cw = myFont != null ? CellsWide : -1;
                int ct = myFont != null ? CellsTall : -1;
                myFont = value;
                if (myParent == null)
                    FontCellSize.Width = Convert.ToInt32(myFont.SizeInPoints) + 1;
                else
                {
                    Graphics G = myParent.MyGraphics;
                    int x1w = Convert.ToInt32(G.MeasureString("X", myFont).Width);
                    int x2w = Convert.ToInt32(G.MeasureString("XX", myFont).Width);
                    FontCellSize.Width = x2w - x1w + 3;
                    //OffsetLeft = (x1w - FontCellSize.Width) / 2;
                }
                FontCellSize.Height = Convert.ToInt32(myFont.SizeInPoints * 1.5F);

                if (cw > -1)
                {
                    Width = cw * CellW;
                    Height = ct * CellH;
                }
                myCachedImage = null;
                if (myParent != null)
                    myParent.Refresh();

            }
        }

        private int myHorizontalCellPadding = 0;
        /// <summary>
        /// Gets or sets the amount of horizontal space added between the cells. Default is 0, can be set to positive or negative values.
        /// </summary>
        public int HorizontalCellPadding
        {
            get
            {
                return myHorizontalCellPadding;
            }
            set
            {
                myHorizontalCellPadding = value;
                OffsetLeft = Convert.ToInt32(Math.Ceiling(value / -2F) + 1);
            }
        }

        private int myVerticalCellPadding = 0;
        /// <summary>
        /// Gets or sets the amount of vertical space added between the cells. Default is 0, can be set to positive or negative values.
        /// </summary>
        public int VerticalCellPadding
        {
            get
            {
                return myVerticalCellPadding;
            }
            set
            {
                myVerticalCellPadding = value;
                OffsetTop = Convert.ToInt32(Math.Ceiling(value / -2F) + 1);
            }
        }

        private int OffsetLeft = 2;
        private int OffsetTop = 0;

        private Size FontCellSize = new Size(0, 0);

        /// <summary>
        /// Clears the internal cache window, and then redraws it.
        /// </summary>
        protected void FlushWindow()
        {
            myCachedImage = null;
            DrawWindow();
        }

        /// <summary>
        /// Dumps the data from all of this window's internal tile storage.
        /// </summary>
        public void ClearCells()
        {
            myDisplay.Clear();
        }

        /// <summary>
        /// Creates and returns a string consisting of the symbols of this window.
        /// </summary>
        /// <returns>String representation of this window's contents.</returns>
        public string DumpToText()
        {
            StringBuilder rVal = new StringBuilder();
            rVal.Append('+');
            rVal.Append(new string('-', CellsWide));
            rVal.Append('+');
            rVal.AppendLine();
            for (int y = 0; y < CellsTall; y++)
            {
                rVal.Append('|');
                for (int x = 0; x < CellsWide; x++)
                {
                    cellDef C = myDisplay[new Point(x, y)];
                    if (C != null)
                        rVal.Append(C.ToText());
                    else
                        rVal.Append(' ');

                }
                rVal.Append('|');
                rVal.AppendLine();
            }
            rVal.Append('+');
            rVal.Append(new string('-', CellsWide));
            rVal.Append('+');
            rVal.AppendLine();

            return rVal.ToString();
        }

        /// <summary>
        /// Draws a string on this window.
        /// </summary>
        /// <param name="S">String to be drawn.</param>
        /// <param name="X">X-coordinate of the starting cell, in cells.</param>
        /// <param name="Y">Y-coordinate of the starting cell, in cells.</param>
        /// <param name="ForeColor">Foreground color of the resulting string.</param>
        /// <param name="BackColor">Background color of the resulting string.</param>
        /// <param name="updateNow">Determines whether or not the display should be updated immediately.</param>
        protected void DrawString(string S, int X, int Y, Color ForeColor, Color BackColor, bool updateNow)
        {
            int InitX = X;
            for (int i = 0; i < S.Length; i++)
            {
                if (S[i] == '\n')
                {
                    Y++;
                    X = InitX;
                    continue;
                }
                SetCell(X, Y, S[i], ForeColor, BackColor, false);
                X++;
            }
            if (updateNow) Update();
        }

        /// <summary>
        /// Causes this window to immediately update the display with its new contents.
        /// </summary>
        public void Update()
        {
            DrawWindow();
        }

        /// <summary>
        /// Causes this window to update the display with a single cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to be updated, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to be updated, in cells.</param>
        public virtual void UpdateCell(int x, int y)
        {
            Point tP = new Point(x, y);
            if (myDisplay.ContainsKey(tP))
            {
                cellDef CD = myDisplay[tP];
                DrawCell(CD, true);
                return;
            }
        }

        internal virtual void DrawCell(cellDef CD, bool updateNow)
        {
            if (myParent == null)
                return;
            if (myCachedImage == null)
                RenderCachedImage();

            Graphics cachedGraphics = Graphics.FromImage(myCachedImage);
            CD.DrawCell(cachedGraphics, Font, 0, 0, CellW, CellH, OffsetLeft, OffsetTop, UseImages);
            RenderHighlight(cachedGraphics);
            if (updateNow && !myParent.drawingLocked)
            {
                int cellOffX = CD.X * CellW;
                int cellOffY = CD.Y * CellH;

                Image tempImage = new Bitmap(CellW, CellH, System.Drawing.Imaging.PixelFormat.Format32bppPArgb);
                Graphics tempGraphics = Graphics.FromImage(tempImage);
                CD.DrawCell(tempGraphics, Font, -cellOffX, -cellOffY, CellW, CellH, OffsetLeft, OffsetTop, UseImages); //X, Y, CellW, CellH, OffsetLeft, UseImages);
                myParent.MyGraphics.DrawImage(tempImage, cellOffX, cellOffY, CellW, CellH);
                RenderHighlight(myParent.MyGraphics);
            }
        }

        private cellDef FetchOrCreateCell(int x, int y)
        {
            cellDef myC = null;
            Point tP = new Point(x, y);
            if (!myDisplay.TryGetValue(tP, out myC))
            {
                myC = new cellDef();
                myC.X = x;
                myC.Y = y;
                myDisplay.Add(tP, myC);
            }

            return myC;
        }

        /// <summary>
        /// Clears the given cell and updates the display.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to be cleared, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to be cleared, in cells.</param>
        public virtual void ClearCell(int x, int y)
        {
            ClearCell(x, y, true);
        }

        /// <summary>
        /// Clears the given cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to be cleared, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to be cleared, in cells.</param>
        /// <param name="updateNow">Determines whether the display should be updated immediately.</param>
        public virtual void ClearCell(int x, int y, bool updateNow)
        {
            Point tP = new Point(x, y);
            if (myDisplay.ContainsKey(tP))
            {
                cellDef CD = myDisplay[tP];
                CD.Clear();
                DrawCell(CD, updateNow);
                return;
            }
        }

        /// <summary>
        /// Sets the contents of the given cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to set, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to set, in cells.</param>
        /// <param name="T">Tile to set this cell to.</param>        
        /// <param name="updateNow">Determines whether or not the display should be updated immediately.</param>
        public virtual void SetCell(int x, int y, Tile T, bool updateNow)
        {
            if (x < 0 || y < 0 || x >= CellsWide || y >= CellsTall)
                return;
            cellDef myC = FetchOrCreateCell(x, y);
            myC.Clear();
            myC.AddTile(T);
            DrawCell(myC, updateNow);
        }

        /// <summary>
        /// Sets the contents of the given cell and updates the display.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to set, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to set, in cells.</param>
        /// <param name="T">Tile to set this cell to.</param>                
        public virtual void SetCell(int x, int y, Tile T)
        {
            SetCell(x, y, T, true);
        }

        /// <summary>
        /// Sets the contents of the given cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to set, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to set, in cells.</param>
        /// <param name="Symbol">Symbol to set this cell with.</param>
        /// <param name="fC">Foreground color to set this cell with.</param>
        /// <param name="bC">Background color to set this cell with.</param>
        /// <param name="updateNow">Determines whether or not the display should be updated immediately.</param>
        public virtual void SetCell(int x, int y, char Symbol, Color fC, Color bC, bool updateNow)
        {
            SetCell(x, y, new Tile(Symbol, fC, bC, null), updateNow);
        }

        /// <summary>
        /// Sets the contents of the given cell and updates the display.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to set, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to set, in cells.</param>
        /// <param name="Symbol">Symbol to set this cell with.</param>
        /// <param name="fC">Foreground color to set this cell with.</param>
        /// <param name="bC">Background color to set this cell with.</param>        
        public void SetCell(int x, int y, char Symbol, Color fC, Color bC)
        {
            SetCell(x, y, Symbol, fC, bC, true);
        }

        /// <summary>
        /// Sets the contents of the given cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to set, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to set, in cells.</param>
        /// <param name="Symbol">Symbol to set this cell with.</param>
        /// <param name="fC">Foreground color to set this cell with.</param>
        /// <param name="updateNow">Determines whether or not the display should be updated immediately.</param>
        public void SetCell(int x, int y, char Symbol, Color fC, bool updateNow)
        {
            SetCell(x, y, Symbol, fC, Color.Black, updateNow);
        }

        /// <summary>
        /// Sets the contents of the given cell and updates the display.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to set, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to set, in cells.</param>
        /// <param name="Symbol">Symbol to set this cell with.</param>
        /// <param name="fC">Foreground color to set this cell with.</param>
        public void SetCell(int x, int y, char Symbol, Color fC)
        {
            SetCell(x, y, Symbol, fC, Color.Black, true);
        }

        /// <summary>
        /// Sets the contents of the given cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to set, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to set, in cells.</param>
        /// <param name="Symbol">Symbol to set this cell with.</param>
        /// <param name="updateNow">Determines whether or not the display should be updated immediately.</param>
        public void SetCell(int x, int y, char Symbol, bool updateNow)
        {
            SetCell(x, y, Symbol, Color.White, Color.Black, updateNow);
        }

        /// <summary>
        /// Sets the contents of the given cell and updates the display.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to set, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to set, in cells.</param>
        /// <param name="Symbol">Symbol to set this cell with.</param>
        public void SetCell(int x, int y, char Symbol)
        {
            SetCell(x, y, Symbol, Color.White, Color.Black, true);
        }

        /// <summary>
        /// Stacks the given tile atop the given cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="T">Tile to stack onto this cell.</param>        
        /// <param name="updateNow">Determines whether or not the display should be updated immediately.</param>
        public virtual void StackCell(int x, int y, Tile T, bool updateNow)
        {
            if (x < 0 || y < 0 || x >= CellsWide || y >= CellsTall)
                return;
            cellDef myC = FetchOrCreateCell(x, y);
            myC.AddTile(T);
            DrawCell(myC, updateNow);
        }

        /// <summary>
        /// Stacks the given tile atop the given cell and updates the display.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="T">Tile to stack onto this cell.</param>        
        public virtual void StackCell(int x, int y, Tile T)
        {
            StackCell(x, y, T, true);
        }

        /// <summary>
        /// Stacks the given tile components atop the given cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="Symbol">Symbol to stack onto the cell.</param>
        /// <param name="fC">Forecolor to use to stack onto the cell.</param>
        /// <param name="bC">Backcolor to use to stack onto the cell.</param>
        /// <param name="updateNow">Determines whether or not the display should be updated immediately.</param>        
        public virtual void StackCell(int x, int y, char Symbol, Color fC, Color bC, bool updateNow)
        {
            StackCell(x, y, new Tile(Symbol, fC, bC, null), updateNow);
        }

        /// <summary>
        /// Stacks the given tile components atop the given cell and updates the display.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="Symbol">Symbol to stack onto the cell.</param>
        /// <param name="fC">Forecolor to use to stack onto the cell.</param>
        /// <param name="bC">Backcolor to use to stack onto the cell.</param>
        public void StackCell(int x, int y, char Symbol, Color fC, Color bC)
        {
            StackCell(x, y, Symbol, fC, bC, true);
        }

        /// <summary>
        /// Stacks the given tile components atop the given cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="Symbol">Symbol to stack onto the cell.</param>
        /// <param name="fC">Forecolor to use to stack onto the cell.</param>
        /// <param name="updateNow">Determines whether or not the display should be updated immediately.</param>    
        public void StackCell(int x, int y, char Symbol, Color fC, bool updateNow)
        {
            StackCell(x, y, Symbol, fC, Color.Black, updateNow);
        }

        /// <summary>
        /// Stacks the given tile components atop the given cell and updates the display.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="Symbol">Symbol to stack onto the cell.</param>
        /// <param name="fC">Forecolor to use to stack onto the cell.</param>
        public void StackCell(int x, int y, char Symbol, Color fC)
        {
            StackCell(x, y, Symbol, fC, Color.Black, true);
        }

        /// <summary>
        /// Stacks the given tile components atop the given cell.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="Symbol">Symbol to stack onto the cell.</param>
        /// <param name="updateNow">Determines whether or not the display should be updated immediately.</param>    
        public void StackCell(int x, int y, char Symbol, bool updateNow)
        {
            StackCell(x, y, Symbol, Color.White, Color.Black, updateNow);
        }

        /// <summary>
        /// Stacks the given tile components atop the given cell and updates the display.
        /// </summary>
        /// <param name="x">X-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="y">Y-coordinate of the cell to stack onto, in cells.</param>
        /// <param name="Symbol">Symbol to stack onto the cell.</param>
        public void StackCell(int x, int y, char Symbol)
        {
            StackCell(x, y, Symbol, Color.White, Color.Black, true);
        }

        /// <summary>
        /// Sets the highlight for this window
        /// </summary>
        /// <param name="x">X coordinate of the rectangle to highlight</param>
        /// <param name="y">Y coordinate of the rectangle to highlight</param>
        /// <param name="width">Width of the rectangle to highlight</param>
        /// <param name="height">Height of the rectangle to highlight</param>
        /// <param name="highlightColor">Color to highlight in</param>
        public virtual void HighlightCellRegion(int x, int y, int width, int height, Color highlightColor)
        {
            if (myHighlight == null ||
                myHighlight.Value.X != x ||
                myHighlight.Value.Y != y ||
                myHighlight.Value.Width != width ||
                myHighlight.Value.Height != height)
            {
                myHighlight = new Rectangle(x, y, width, height);
                myHighlightColor = highlightColor;
                myCachedImage = null;
                DrawWindow();
            }
        }

        /// <summary>
        /// Removes all highlighting from this window.
        /// </summary>
        public void ClearHighlight()
        {
            if (myHighlight != null)
            {
                myHighlight = null;
                myHighlightColor = null;
                myCachedImage = null;
                DrawWindow();
            }
        }
    }
}
