using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Phasma
{
    /// <summary>
    /// Class that describes the visual contents of a single tile. A cell on the display can consist of a single tile, or of a stack of tiles layered atop one another.
    /// This class will automatically create a pseudo-image by drawing and scaling its symbol, if it is asked to be drawn as an image but has not been given an actual image to use.
    /// </summary>
    public class Tile
    {
        private char mySymbol;
        /// <summary>
        /// Gets or sets the symbol that this tile displays as.
        /// </summary>
        public char Symbol
        {
            get { return mySymbol; }
            set { mySymbol = value; }
        }


        private Image image = null;
        private Image cachedDummy = null;

        internal SolidBrush foreBrush = new SolidBrush(Color.Black), backBrush = new SolidBrush(Color.Black);

        /// <summary>
        /// Gets or sets the foreground color that this tile uses.
        /// </summary>
        public Color ForeColor
        {
            get { return foreBrush.Color; }
            set { foreBrush.Color = value; }
        }

        /// <summary>
        /// Gets or sets the background color that this tile uses.
        /// </summary>
        public Color BackColor
        {
            get { return backBrush.Color; }
            set { backBrush.Color = value; }
        }

        /// <summary>
        /// Constructor for Tile
        /// </summary>
        /// <param name="S">Symbol to draw with.</param>
        public Tile(char S)
            : this(S, Color.White, Color.Black, null)
        {
        }

        /// <summary>
        /// Constructor for Tile
        /// </summary>
        /// <param name="S">Symbol to draw with.</param>
        /// <param name="FC">Foreground color to draw in.</param>
        public Tile(char S, Color FC)
            : this(S, FC, Color.Black, null)
        {
        }

        /// <summary>
        /// Constructor for Tile
        /// </summary>
        /// <param name="S">Symbol to draw with</param>
        /// <param name="FC">Foreground color to draw in.</param>
        /// <param name="I">Image to draw with.</param>
        public Tile(char S, Color FC, Image I)
            : this(S, FC, Color.Black, I)
        {
        }


        /// <summary>
        /// Constructor for Tile
        /// </summary>
        /// <param name="S">Symbol to draw with</param>
        /// <param name="FC">Foreground color to draw in.</param>
        /// <param name="BC">Background color to draw in.</param>
        public Tile(char S, Color FC, Color BC)
            : this(S, FC, BC, null)
        {
        }

        /// <summary>
        /// Constructor for Tile
        /// </summary>
        /// <param name="S">Symbol to draw with</param>
        /// <param name="FC">Foreground color to draw in</param>
        /// <param name="BC">Background color to draw in.</param>
        /// <param name="I">Image to draw with.</param>
        public Tile(char S, Color FC, Color BC, Image I)
        {
            Symbol = S;
            ForeColor = FC;
            BackColor = BC;
            image = I;
        }

        /// <summary>
        /// Gets or sets the image of this tile.
        /// </summary>
        public Image Image
        {
            get
            {
                if (image != null)
                    return image;

                if (cachedDummy != null)
                    return cachedDummy;

                cachedDummy = new Bitmap(32, 32);
                Graphics G = Graphics.FromImage(cachedDummy);
                G.FillRectangle(backBrush, 0, 0, 32, 32);
                G.DrawString(Symbol.ToString(), new Font("Courier New", 18, FontStyle.Bold), foreBrush, 0, 0);
                return cachedDummy;
            }
            set
            {
                image = value;
            }
        }


    }
}