using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Phasma
{
    /// <summary>
    /// Class describing a single piece of data to be output on an RLStats object.
    /// </summary>
    public class statDef
    {
        /// <summary>
        /// Basic constructor for a statDef object.
        /// </summary>
        /// <param name="iK">Key of the new statDef, used to make changes to it from the RLStats object.</param>
        /// <param name="iST">Sets what kind of display the new statDef will use.</param>
        /// <param name="iX">X-coordinate for the new statDef, in cells, within the RLStats object.</param>
        /// <param name="iY">Y-coordinate for the new statDef, in cells, within the RLStats object.</param>
        public statDef(string iK, StatTypes iST, int iX, int iY)
        {
            myKey = iK;
            myStatType = iST;
            myX = iX;
            myY = iY;
        }

        /// <summary>
        /// Different types of displays that statDefs can use.
        /// </summary>
        public enum StatTypes
        {
            /// <summary>
            /// Key of the statDef, then the value.
            /// </summary>
            Output,            
            /// <summary>
            /// Just the value of the statDef.
            /// </summary>
            KeylessOutput,
            /// <summary>
            /// Status bar, using the value to set the amount filled.
            /// </summary>
            Bar,
            /// <summary>
            /// List of binary flags.
            /// </summary>
            FlagList
        }

        protected Color myColor1 = Color.White;
        /// <summary>
        /// Gets or sets the main color of the statDef.
        /// </summary>
        public Color Color1
        {
            get { return myColor1; }
            set { if (myColor1 == value) return; RaiseClear(); myColor1 = value; RaiseDraw(); }
        }

        protected Color myColor2 = Color.White;
        /// <summary>
        /// Gets or sets the secondary color of the statDef.
        /// </summary>
        public Color Color2
        {
            get { return myColor2; }
            set { if (myColor2 == value) return; RaiseClear(); myColor2 = value; RaiseDraw(); }
        }

        private StatTypes myStatType = StatTypes.Output;
        /// <summary>
        /// Gets or sets the display type of the statDef.
        /// </summary>
        public StatTypes StatType
        {
            get { return myStatType; }
            set { if (myStatType == value) return; RaiseClear(); myStatType = value; RaiseDraw(); }
        }

        private int myX = 0;
        /// <summary>
        /// Gets or sets the x-coordinate (in cells, relative to the RLStats) of the statDef.
        /// </summary>
        public int X
        {
            get { return myX; }
            set { if (myX == value) return; RaiseClear(); myX = value; RaiseDraw(); }
        }

        private int myY = 0;
        /// <summary>
        /// Gets or sets the y-coordinate (in cells, relative to the RLStats) of the statDef.
        /// </summary>
        public int Y
        {
            get { return myY; }
            set { if (myY == value) return; RaiseClear(); myY = value; RaiseDraw(); }
        }

        private bool myVisible = true;
        /// <summary>
        /// Gets or sets whether this statDef should currently be displayed.
        /// </summary>
        public bool Visible
        {
            get { return myVisible; }
            set { if (myVisible == value) return; RaiseClear(); myVisible = value; RaiseDraw(); }
        }

        private string myValue = "";
        /// <summary>
        /// Gets or sets the value of the statDef.
        /// If this statDef is a Bar, this should be a string of the form "part/whole", where part is a number representing
        /// the current amount, and whole is a number representing the maximum amount.
        /// </summary>
        public string Value
        {
            get { return myValue; }
            set { if (myValue == value) return; RaiseClear(); myValue = value; RaiseDraw(); }
        }

        private int myLength = -1;
        /// <summary>
        /// Gets or sets the total number of cells, horizontally, that the statDef takes up. If this is set to a negative value, it will be auto-sized.
        /// </summary>
        public virtual int Length
        {
            get
            {
                if (myLength < 0)
                    if (StatType == StatTypes.Output)
                        return Key.Length + Value.Length + 2;
                    else if (StatType == StatTypes.KeylessOutput)
                        return Value.Length;
                    else if (StatType == StatTypes.Bar)
                        return Key.Length + Value.Length + 17;
                return myLength;
            }
            set { if (myLength == value) return; RaiseClear(); myLength = value; RaiseDraw(); }
        }

        private string myKey = "";
        /// <summary>
        /// Returns the key associated with this statDef.
        /// </summary>
        public string Key
        {
            get { return myKey; }
        }

        internal delegate void statEvent(string outKey);
        internal event statEvent ClearMe;
        internal event statEvent DrawMe;

        protected void RaiseDraw()
        {
            if (DrawMe != null)
                DrawMe(Key);
        }

        protected void RaiseClear()
        {
            if (ClearMe != null)
                ClearMe(Key);
        }

    }

    /// <summary>
    /// A statDef that is capable of displaying a list of binary flags.
    /// </summary>
    public class flagListDef : statDef
    {
        /// <summary>
        /// Basic constructor for a flagListDef.
        /// </summary>
        /// <param name="iK">Key of the new flagListDef, used to make changes to it from the RLStats object.</param>        
        /// <param name="iX">X-coordinate for the new flagListDef, in cells, within the RLStats object.</param>
        /// <param name="iY">Y-coordinate for the new flagListDef, in cells, within the RLStats object.</param>
        public flagListDef(string iK, int iX, int iY)
            : base(iK, StatTypes.FlagList, iX, iY)
        { }

        internal List<statDef> Flags = new List<statDef>();

        /// <summary>
        /// Adds a new flag to the list of flags that this flagListDef can display.
        /// </summary>
        /// <param name="fKey">Key of the new flag.</param>
        /// <param name="fVal">Displayed text for the new flag.</param>
        /// <param name="fClr">Color of the new flag.</param>
        /// <param name="fVis">Initial visibility of the new flag.</param>
        public void AddFlag(string fKey, string fVal, Color fClr, bool fVis)
        {
            RaiseClear();
            statDef D = new statDef(fKey, StatTypes.KeylessOutput, 0, 0);
            D.Value = fVal;
            D.Color1 = fClr;
            D.Visible = fVis;
            Flags.Add(D);
            RaiseDraw();            
        }      

        /// <summary>
        /// Removes the flag with a given key from the list of flags that this flagListDef can display.
        /// </summary>
        /// <param name="fKey">Key of the flag to be removed.</param>
        public void RemFlag(string fKey)
        {
            RaiseClear();
            foreach (statDef S in Flags)
                if (S.Key == fKey)
                {
                    Flags.Remove(S);
                    break;
                }
            RaiseDraw();
        }

        /// <summary>
        /// Sets the visibility of the flag with the given key.
        /// </summary>
        /// <param name="fKey">Key of the flag whose visibility should be set.</param>
        /// <param name="fVis">New visibility for the given flag.</param>
        public void SetFlag(string fKey, bool fVis)
        {
            RaiseClear();
            foreach (statDef S in Flags)
                if (S.Key == fKey)
                {
                    S.Visible = fVis;
                    break;
                }
            RaiseDraw();
        }

        /// <summary>
        /// Returns the visibility of the flag with the given key.
        /// </summary>
        /// <param name="fKey">Key of the flag whose visibility should be returned.</param>
        /// <returns>Whether or not the flag is visible.</returns>
        public bool GetVisibility(string fKey)
        {
            foreach (statDef S in Flags)
                if (S.Key == fKey)
                {
                    return S.Visible;
                }
            return false;
        }
    }
}
